SystemOrganization addCategory: #'CS520-HomeAutomation'!

TestCase subclass: #HomeAutomationTest
	instanceVariableNames: 'kitchen livingRoom alice bob'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!HomeAutomationTest methodsFor: 'running' stamp: 'CameronRieck 3/14/2013 21:32'!
setUp
	alice := User createWithName: 'Alice'.
	bob := User createWithName: 'Bob'.
	kitchen := Room createWithName: 'Kitchen'.
	livingRoom := Room createWithName: 'Living Room'.! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:40'!
testAcitivtyTrackerDoesNotGetNotfiedOfUserEntry
	| device user acitivtyTracker |
	device := HomeAutomationDevice dummy.
	device attachTo: kitchen.
	acitivtyTracker := ActivityTracker new.
	acitivtyTracker observe: device.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	self deny: (acitivtyTracker detectedUser: user).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:40'!
testAcitivtyTrackerDoesNotGetNotfiedOfUserExitAfterStopsObserving
	| device user acitivtyTracker |
	device := Camera new.
	device attachTo: kitchen.
	acitivtyTracker := ActivityTracker new.
	acitivtyTracker observe: device.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	acitivtyTracker stopObserving: device.
	user exitCurrentRoom.
	self assert: (acitivtyTracker detectedUser: user).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:40'!
testAcitivtyTrackerGetsNotfiedOfUserEntry
	| device user acitivtyTracker |
	device := Camera new.
	device attachTo: kitchen.
	acitivtyTracker := ActivityTracker new.
	acitivtyTracker observe: device.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	self assert: (acitivtyTracker detectedUser: user).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:41'!
testAcitivtyTrackerGetsNotfiedOfUserEntryWithRoom
	| device user acitivtyTracker |
	device := Camera new.
	device attachTo: kitchen.
	acitivtyTracker := ActivityTracker new.
	acitivtyTracker observe: device.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	self assert: (acitivtyTracker detectedUser: user).
	self assert: ((user room) = kitchen)! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:41'!
testAcitivtyTrackerGetsNotfiedOfUserExit
	| device user acitivtyTracker |
	device := Camera new.
	device attachTo: kitchen.
	acitivtyTracker := ActivityTracker new.
	acitivtyTracker observe: device.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	self assert: (acitivtyTracker detectedUser: user).
	self assert: (user room = kitchen).
	user exitCurrentRoom.
	self deny: (acitivtyTracker detectedUser: user).
	self assert: (user room = Room noneRoom).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:41'!
testAttachDeviceToRoom
	| device |
	device := Camera new.
	device attachTo: kitchen.
	
	self assert: (kitchen hasDevice: device).
	self assert: (device isAttachedTo: kitchen).
	self assert: (device isAttachedToARoom).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 21:10'!
testCreateCameraSensor
	| camera |
	camera := Camera createWithName: 'Camera 01'.
	self assert: (camera name = 'Camera 01').! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:44'!
testCreateRoom
	| room |
	room := Room createWithName: 'Kitchen'.
	self assert: (room name = 'Kitchen').! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/14/2013 22:19'!
testCreateSmartDoor
	| door |
	door := SmartDoor createWithName: 'Kitchen Door'.
	door allowUser: alice.
	door allowUser: bob.
	self assert: (door name = 'Kitchen Door').
	self assert: (door isUnlocked).
	self assert: (door allowedUsers size = 2).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/13/2013 20:12'!
testCreateSmartLight
	| light |
	light := SmartLight createWithName: 'L1'.
	self assert: (light isOff).
	self assert: (light room = Room noneRoom).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/13/2013 20:14'!
testCreateSmartThermostat
	| thermostat |
	thermostat := SmartThermostat createWithName: 'T1'.
	self assert: (thermostat name = 'T1').
	self assert: (thermostat room = Room noneRoom).
	self assert: (thermostat temperature = thermostat defaultTemperature).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/10/2013 17:56'!
testCreateUser
	| user |
	user := User createWithName: 'simon'.
	self assert: (user name = 'simon').
	self assert: (user room = Room noneRoom).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:42'!
testDetachDeviceFromRoom
	| device |
	device := Camera new.
	device attachTo: kitchen.
	device detachFromRoom.
	self deny: (kitchen hasDevice: device).
	self deny: (device isAttachedTo: kitchen).
	self assert: (device isAttachedTo: Room noneRoom).
	self deny: (device isAttachedToARoom).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:42'!
testHomeCommunicatorSendsMessageToUser
	|  kSpeaker lrSpeaker kMic lrMic kCam lrCam tom alice homeCommunicator |
	kSpeaker := Speaker createWithName: 'S1'.
	lrSpeaker := Speaker createWithName: 'S2'.
	kMic := Microphone createWithName: 'M1'.
	lrMic := Microphone createWithName: 'M2'.
	kCam := Camera createWithName: 'C1'.
	lrCam := Camera createWithName: 'C2'.
	homeCommunicator := HomeCommunicator createWithName: 'HC'.
	tom := User createWithName: 'Tom'.
	alice := User createWithName: 'Alice'.
	
	homeCommunicator observe: lrMic.
	homeCommunicator observe: kMic.
	homeCommunicator observe: lrCam.
	homeCommunicator observe: kCam.
	homeCommunicator useSpeaker: lrSpeaker.
	homeCommunicator useSpeaker: kSpeaker.
	
	lrMic attachTo: livingRoom.
	lrSpeaker attachTo: livingRoom.
	lrCam attachTo: livingRoom.
	kMic attachTo: kitchen.
	kSpeaker attachTo: kitchen.
	lrCam attachTo: kitchen.

	tom enterRoom: kitchen.
	alice enterRoom: livingRoom.
	
	tom speak: 'Alice: are you hungry?'.
	self assert: (alice lastNoiseHeard = 'Alice: are you hungry?').! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/14/2013 22:11'!
testSmartDoorAllowsUserToReEnter
	| door |
	door := SmartDoor createWithName: 'Kitchen Door'.
	door attachTo: kitchen.
	alice enterRoom: kitchen.
	door lock.
	alice exitCurrentRoom.
	self assert: (alice room = Room noneRoom).
	alice enterRoom: kitchen.
	self assert: (alice room = kitchen).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/14/2013 22:12'!
testSmartDoorKeepsOutUser
	| door |
	door := SmartDoor createWithName: 'Kitchen Door'.
	door attachTo: kitchen.
	alice enterRoom: kitchen.
	door lock.
	self assert: (door isLocked).
	bob enterRoom: kitchen.
	self assert: (alice room = kitchen).
	self assert: (bob room = Room noneRoom).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/13/2013 20:18'!
testSmartLightTurnsOffWhenUserExits
	| camera light user |
	camera := Camera createWithName: 'C1'.
	camera attachTo: kitchen.
	light := SmartLight createWithName: 'L1'.
	light observe: camera.
	light attachTo: kitchen.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	user exitCurrentRoom.
	self assert: (light isOff).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/13/2013 20:18'!
testSmartLightTurnsOnWhenUserEnters
	| camera light user |
	camera := Camera createWithName: 'C1'.
	camera attachTo: kitchen.
	light := SmartLight createWithName: 'L1'.
	light observe: camera.
	light attachTo: kitchen.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	self assert: (light isOn).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/13/2013 20:20'!
testSmartThermostatChangesToSingleUsersPreference
	| camera thermostat user |
	camera := Camera createWithName: 'C1'.
	camera attachTo: kitchen.
	thermostat := SmartThermostat createWithName: 'T1'.
	thermostat observe: camera.
	thermostat attachTo: kitchen.
	user := User createWithName: 'Tom'.
	user preferredTemperature: 65.
	user enterRoom: kitchen.
	self assert: (thermostat temperature = 65).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/13/2013 20:30'!
testSmartThermostatChangesToTwoUsersPreferences
	| camera thermostat user1 user2 |
	camera := Camera createWithName: 'C1'.
	camera attachTo: kitchen.
	thermostat := SmartThermostat createWithName: 'T1'.
	thermostat observe: camera.
	thermostat attachTo: kitchen.
	user1 := User createWithName: 'Tom'.
	user1 preferredTemperature: 77.
	user1 enterRoom: kitchen.
	user2 := User createWithName: 'Alice'.
	user2 preferredTemperature: 66.
	user2 enterRoom: kitchen.
	self assert: (thermostat temperature = 71).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:43'!
testSpeakerSendsOutputToRoom
	| speaker user observer |
	speaker := Speaker createWithName: 'S1'.
	speaker attachTo: kitchen.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	speaker broadcast: 'this is a test'.
	self assert: (user lastNoiseHeard = 'this is a test').
	self assert: (user lastNoiseHeardSource = speaker).
	! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:43'!
testSpeakerSendsOutputToRoomAfterUserExits
	| speaker user observer |
	speaker := Speaker createWithName: 'S1'.
	speaker attachTo: kitchen.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	user exitCurrentRoom.
	speaker broadcast: 'this is a test'.
	self deny: (user lastNoiseHeard = 'this is a test').
	self deny: (user lastNoiseHeardSource = speaker).
	! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:43'!
testUserEntersRoom
	| user |
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	self assert: (kitchen includesUser: user).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:43'!
testUserEntersRoomBeforeDevicesAttached
	| user device observer |
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.

	device := Camera new.
	observer := ActivityTracker new.
	observer observe: device.
	device attachTo: kitchen.
	self assert: (observer detectedUser: user).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:43'!
testUserSpeaks
	| microphone user observer |
	microphone := Microphone createWithName: 'M1'.
	microphone attachTo: kitchen.
	observer := ActivityTracker new.
	observer observe: microphone.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	user speak: 'hello everyone!!'.
	self assert: (observer hasHeard: 'hello everyone!!' from: user).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:44'!
testUserSpeaksInRoomWithoutMicrophone
	| microphone user observer |
	microphone := Microphone createWithName: 'M1'.
	microphone attachTo: kitchen.
	observer := ActivityTracker new.
	observer observe: microphone.
	user := User createWithName: 'Tom'.
	user enterRoom: livingRoom.
	user speak: 'hello everyone!!'.
	self deny: (observer hasHeard: 'hello everyone!!' from: user).! !

!HomeAutomationTest methodsFor: 'tests' stamp: 'CameronRieck 3/12/2013 19:44'!
testUserSpeaksInRoomWithoutMicrophoneAfterLeaving
	| microphone user observer |
	microphone := Microphone createWithName: 'M1'.
	microphone attachTo: kitchen.
	observer := ActivityTracker new.
	observer observe: microphone.
	user := User createWithName: 'Tom'.
	user enterRoom: kitchen.
	user exitCurrentRoom.
	user enterRoom: livingRoom.
	user speak: 'hello everyone!!'.
	self deny: (observer hasHeard: 'hello everyone!!' from: user).! !

RectangleMorph subclass: #HomeAutomationView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

HomeAutomationView subclass: #DeviceView
	instanceVariableNames: 'device label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

DeviceView subclass: #ActivityTrackerDeviceView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!ActivityTrackerDeviceView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/15/2013 19:22'!
createFromDevice: aDevice
	| newDeviceView |
	newDeviceView := ActivityTrackerDeviceView new.
	newDeviceView color: (newDeviceView defaultColor).
	newDeviceView height: (newDeviceView defaultHeight) width: (newDeviceView defaultWidth).
	newDeviceView device: aDevice.
	^ newDeviceView.! !

!ActivityTrackerDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 20:59'!
defaultColor
	^ Color lightBlue! !

!ActivityTrackerDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/14/2013 20:13'!
highlightColor
	^ Color magenta! !

!ActivityTrackerDeviceView methodsFor: 'events' stamp: 'CameronRieck 3/15/2013 19:26'!
modelUpdated: statusMessage
	super modelUpdated: statusMessage.
	Transcript show: statusMessage; cr.! !

!ActivityTrackerDeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/15/2013 19:26'!
update
	super update.
	self device isTrackingUsers ifTrue: [ self color: self highlightColor ] ifFalse: [ self color: self defaultColor ].! !

!DeviceView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/15/2013 19:28'!
createFromDevice: aDevice
	| newDeviceView |
	newDeviceView := DeviceView new.
	newDeviceView color: (newDeviceView defaultColor).
	newDeviceView height: (newDeviceView defaultHeight) width: (newDeviceView defaultWidth).
	newDeviceView device: aDevice.
	^ newDeviceView.! !

!DeviceView methodsFor: 'submorphs' stamp: 'CameronRieck 3/12/2013 21:24'!
addLabel
	label contents: device name.
	self addMorphCentered: label.! !

!DeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 20:59'!
defaultColor
	^ Color black! !

!DeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 21:32'!
defaultHeight
	^ 30! !

!DeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 21:32'!
defaultWidth
	^ 30! !

!DeviceView methodsFor: 'model' stamp: 'CameronRieck 3/9/2013 23:55'!
device
	^ device! !

!DeviceView methodsFor: 'model' stamp: 'CameronRieck 3/15/2013 19:28'!
device: aDevice 
	device := aDevice.
	device when: #updated send: #modelUpdated: to: self with: #statusMessage.
	self removeAllMorphs.
	self addLabel.
	self updateColors.! !

!DeviceView methodsFor: 'initialize-release' stamp: 'CameronRieck 3/12/2013 21:31'!
initialize
	super initialize.
	label := StringMorph new.! !

!DeviceView methodsFor: 'testing' stamp: 'CameronRieck 3/10/2013 00:00'!
isDeviceView
	^ true! !

!DeviceView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/16/2013 15:27'!
justDroppedInto: aPotentialRoomView event: anEvent
	super justDroppedInto: aPotentialRoomView event: anEvent.
	aPotentialRoomView isRoomView
		ifTrue: [ self device attachTo: aPotentialRoomView room ]! !

!DeviceView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/11/2013 13:35'!
justGrabbedFrom: formerOwner
	super justGrabbedFrom: formerOwner.
	formerOwner isRoomView
		ifTrue: [ self device detachFromRoom ]! !

!DeviceView methodsFor: 'events' stamp: 'CameronRieck 3/12/2013 21:37'!
modelUpdated: aStatusMessage
	self update.! !

!DeviceView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/16/2013 15:26'!
rejectDropMorphEvent: evt
	self formerOwner: previousRoomView.
	super rejectDropMorphEvent: evt.! !

!DeviceView methodsFor: 'submorphs' stamp: 'CameronRieck 3/15/2013 19:00'!
removeLabel
	self removeMorph: label.! !

!DeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/15/2013 18:58'!
update
	self updateColors.! !

!DeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/12/2013 21:24'!
updateColors
	label color: (Color black orColorUnlike: self color).
	self borderColor: label color! !

DeviceView subclass: #SmartDoorDeviceView
	instanceVariableNames: 'lockButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!SmartDoorDeviceView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/15/2013 19:22'!
createFromDevice: aDevice
	| newDeviceView |
	newDeviceView := SmartDoorDeviceView new.
	newDeviceView color: (newDeviceView defaultColor).
	newDeviceView height: (newDeviceView defaultHeight) width: (newDeviceView defaultWidth).
	newDeviceView device: aDevice.
	^ newDeviceView.! !

!SmartDoorDeviceView methodsFor: 'initialize-release' stamp: 'CameronRieck 3/16/2013 14:02'!
createLockButton
	lockButton := SimpleButtonMorph newWithLabel: 'lock'.
	lockButton target: self; actionSelector: #lockButtonPressed.
	self addMorphCentered: lockButton.! !

!SmartDoorDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/15/2013 18:42'!
defaultHeight
	^ 25! !

!SmartDoorDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/15/2013 18:42'!
defaultWidth
	^ 60! !

!SmartDoorDeviceView methodsFor: 'model' stamp: 'CameronRieck 3/15/2013 19:39'!
device: aDevice
	super device: aDevice.
	self removeLabel.
	self createLockButton.
	self updateLockButtonLabel.
	self updateBorderColor.! !

!SmartDoorDeviceView methodsFor: 'initialize-release' stamp: 'CameronRieck 3/15/2013 19:27'!
initialize
	super initialize.! !

!SmartDoorDeviceView methodsFor: 'events' stamp: 'CameronRieck 3/15/2013 19:36'!
lockButtonPressed
	device ifLocked: [ device unlock ] ifUnlocked: [ device lock ]! !

!SmartDoorDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/15/2013 19:38'!
lockedColor
	 ^ Color red! !

!SmartDoorDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/15/2013 19:38'!
unlockedColor
	 ^ Color green! !

!SmartDoorDeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/15/2013 19:39'!
update
	super update.
	self updateLockButtonLabel.
	self updateBorderColor.! !

!SmartDoorDeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/15/2013 19:39'!
updateBorderColor
	device ifLocked: [ self borderColor: self lockedColor ] ifUnlocked: [ self borderColor: self unlockedColor ]! !

!SmartDoorDeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/15/2013 19:40'!
updateLockButtonLabel
	device ifLocked: [ lockButton label: 'unlock' ] ifUnlocked: [ lockButton label: 'lock' ].
	lockButton center: self center.! !

DeviceView subclass: #SmartLightDeviceView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!SmartLightDeviceView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/15/2013 19:22'!
createFromDevice: aDevice
	| newDeviceView |
	newDeviceView := SmartLightDeviceView new.
	newDeviceView color: (newDeviceView defaultColor).
	newDeviceView height: (newDeviceView defaultHeight) width: (newDeviceView defaultWidth).
	newDeviceView device: aDevice.
	^ newDeviceView.! !

!SmartLightDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/13/2013 19:44'!
defaultColor
	^ Color darkGray! !

!SmartLightDeviceView methodsFor: 'model' stamp: 'CameronRieck 3/15/2013 19:24'!
device: aDevice
	super device: aDevice.
	self update.! !

!SmartLightDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/13/2013 19:53'!
offColor
	^ self defaultColor! !

!SmartLightDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/13/2013 19:52'!
onColor
	^ Color yellow! !

!SmartLightDeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/15/2013 19:09'!
update
	super update.
	device ifOn: [ self color: self onColor ] ifOff: [ self color: self offColor ].! !

DeviceView subclass: #SmartThermostatDeviceView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!SmartThermostatDeviceView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/15/2013 19:22'!
createFromDevice: aDevice
	| newDeviceView |
	newDeviceView := SmartThermostatDeviceView new.
	newDeviceView color: (newDeviceView defaultColor).
	newDeviceView height: (newDeviceView defaultHeight) width: (newDeviceView defaultWidth).
	newDeviceView device: aDevice.
	^ newDeviceView.! !

!SmartThermostatDeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/13/2013 20:51'!
adjustAppearance
	label contents: device temperature asString.
	self color: (self colorForTemperature: device temperature).
! !

!SmartThermostatDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/13/2013 20:38'!
coldColor
	^ Color lightBlue ! !

!SmartThermostatDeviceView methodsFor: 'private' stamp: 'CameronRieck 3/13/2013 21:39'!
colorForTemperature: temperature
	(temperature < 60) ifTrue: [ ^ self coldColor ].
	((60 to: 75) includes: temperature) ifTrue: [ ^ self defaultColor ].
	((75 to: 85) includes: temperature) ifTrue: [ ^ self warmColor ].
	(temperature > 85) ifTrue: [ ^ self hotColor ].
	^ self defaultColor.! !

!SmartThermostatDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/13/2013 20:37'!
defaultColor
	^ Color white! !

!SmartThermostatDeviceView methodsFor: 'model' stamp: 'CameronRieck 3/15/2013 19:24'!
device: aDevice
	super device: aDevice.
	self adjustAppearance.
	
! !

!SmartThermostatDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/13/2013 20:39'!
hotColor
	^ Color red! !

!SmartThermostatDeviceView methodsFor: 'updating' stamp: 'CameronRieck 3/15/2013 19:10'!
update
	super update.
	self adjustAppearance.! !

!SmartThermostatDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/13/2013 20:39'!
warmColor
	^ Color orange! !

DeviceView subclass: #SpeakerDeviceView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!SpeakerDeviceView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/15/2013 19:23'!
createFromDevice: aDevice
	| newDeviceView |
	newDeviceView := SpeakerDeviceView new.
	newDeviceView color: (newDeviceView defaultColor).
	newDeviceView height: (newDeviceView defaultHeight) width: (newDeviceView defaultWidth).
	newDeviceView device: aDevice.
	^ newDeviceView.! !

!SpeakerDeviceView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 21:34'!
defaultColor
	^ Color brown! !

!SpeakerDeviceView methodsFor: 'model' stamp: 'CameronRieck 3/12/2013 21:33'!
device: aDevice
	super device: aDevice.
	device when: #speakerNoise send: #speakerMadeNoise: to: self with: #deviceAndWords.! !

!SpeakerDeviceView methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 22:16'!
speakerMadeNoise: deviceAndWords
	self showBalloon: deviceAndWords value , ' '.! !

!HomeAutomationView methodsFor: 'geometry' stamp: 'CameronRieck 3/15/2013 19:16'!
height: aHeight width: aWidth 
	self height: aHeight.
	self width: aWidth.! !

!HomeAutomationView methodsFor: 'testing' stamp: 'CameronRieck 3/7/2013 17:34'!
isHomeAutomationView
	^ true! !

HomeAutomationView subclass: #RoomView
	instanceVariableNames: 'room'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!RoomView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/12/2013 19:57'!
createFromRoom: aRoom
	^ RoomView createFromRoom: aRoom withColor: RoomView defaultColor withSize: RoomView defaultSize! !

!RoomView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/12/2013 20:27'!
createFromRoom: aRoom withColor: aColor
	^ RoomView createFromRoom: aRoom withColor: aColor withWidth: RoomView defaultWidth withHeight: RoomView defaultHeight! !

!RoomView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/12/2013 20:22'!
createFromRoom: aRoom withColor: aColor withWidth: width withHeight: height
	| newRoomView |
	newRoomView := RoomView new.
	newRoomView room: aRoom.
	newRoomView color: aColor.
	newRoomView width: width.
	newRoomView height: height.
	newRoomView update.
	^ newRoomView.! !

!RoomView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/12/2013 20:23'!
createFromRoom: aRoom withWidth: width withHeight: height
	^ RoomView createFromRoom: aRoom withColor: RoomView defaultColor withWidth: width withHeight: height! !

!RoomView methodsFor: 'properties' stamp: 'CameronRieck 3/16/2013 14:18'!
defaultColor
	^ Color gray! !

!RoomView methodsFor: 'properties' stamp: 'CameronRieck 3/16/2013 14:18'!
defaultHeight
	^ 120! !

!RoomView methodsFor: 'properties' stamp: 'CameronRieck 3/16/2013 14:18'!
defaultWidth
	^ 80! !

!RoomView methodsFor: 'devices' stamp: 'CameronRieck 3/9/2013 23:49'!
installDevice: each atPosition: currPosition
	| newDeviceView |
	newDeviceView := DeviceView createFromDevice: each.
	newDeviceView position: currPosition.
	self addMorphFront: newDeviceView.
	^ newDeviceView! !

!RoomView methodsFor: 'devices' stamp: 'CameronRieck 3/9/2013 23:49'!
installDevices
	| currPosition hGap vGap |
	hGap := 5.
	vGap := 5.
	currPosition := hGap @ vGap.
	room attachedDevices
		do: [ :each | 
			| newDeviceView |
			self width > (currPosition x + each width + hGap)
				ifTrue: [ 
					newDeviceView := self installDevice: each atPosition: currPosition.
					currPosition := currPosition + ((newDeviceView width + hGap) @ 0) ] ]! !

!RoomView methodsFor: 'testing' stamp: 'CameronRieck 3/7/2013 16:54'!
isRoomView
	^ true! !

!RoomView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/16/2013 15:24'!
repelsMorph: aMorph event: ev
	^ (self wantsDroppedMorph: aMorph event: ev) not! !

!RoomView methodsFor: 'updating' stamp: 'CameronRieck 3/12/2013 20:30'!
resetViewFromModel
	self removeAllMorphs.
	self setLabelProperties.
	self installDevices.! !

!RoomView methodsFor: 'model' stamp: 'CameronRieck 3/10/2013 00:02'!
room
	^ room! !

!RoomView methodsFor: 'model' stamp: 'CameronRieck 3/12/2013 20:29'!
room: aRoom
	room := aRoom.! !

!RoomView methodsFor: 'updating' stamp: 'CameronRieck 3/4/2013 22:19'!
setLabelProperties
	| label |
	label := StringMorph new.
	label contents: room name.
	label color: (Color black orColorUnlike: self color).
	self borderColor: label color.
	self addMorphCentered: label! !

!RoomView methodsFor: 'updating' stamp: 'CameronRieck 3/12/2013 19:54'!
update
	self resetViewFromModel.! !

!RoomView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/15/2013 19:45'!
wantsDroppedMorph: aMorph event: anEvent
	(aMorph isDeviceView) ifTrue: [ ^ true ].
	(aMorph isUserView) ifTrue: [ ^ room userIsAllowedToEnter: aMorph user ].
	^ false.! !

HomeAutomationView subclass: #UserView
	instanceVariableNames: 'user previousRoomView previousOwner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!UserView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/15/2013 19:18'!
createFromUser: aUser
	| newUserView |
	newUserView := UserView new.
	newUserView color: newUserView defaultColor.
	newUserView user: aUser.
	newUserView height: newUserView defaultHeight width: newUserView defaultWidth.
	^ newUserView.! !

!UserView class methodsFor: 'instance creation' stamp: 'CameronRieck 3/12/2013 21:03'!
createFromUser: aUser withColor: aColor
	| newUserView |
	newUserView := UserView createFromUser: aUser.
	newUserView color: aColor.
	newUserView update.
	^ newUserView.! !

!UserView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 20:44'!
addLabel
	| label |
	label := StringMorph new.
	label contents: user name.
	label color: (Color yellow orColorUnlike: self color).
	self borderColor: label color.
	self addMorphCentered: label! !

!UserView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 20:47'!
addSpeakButton
	| speakButton |
	speakButton := SimpleButtonMorph newWithLabel: 'speak'.
	speakButton target: self; actionSelector: #speakButtonPressed.
	self addMorphCentered: speakButton.
	speakButton position: (speakButton position + (0@25)).! !

!UserView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 21:00'!
defaultColor
	^ Color blue! !

!UserView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 21:01'!
defaultHeight
	^ 80! !

!UserView methodsFor: 'properties' stamp: 'CameronRieck 3/12/2013 21:01'!
defaultWidth
	^ 55! !

!UserView methodsFor: 'properties' stamp: 'CameronRieck 3/15/2013 19:18'!
height: aHeight width: aWidth
	super height: aHeight width: aWidth.
	self removeAllMorphs.
	self addLabel.
	self addSpeakButton.! !

!UserView methodsFor: 'initialize-release' stamp: 'CameronRieck 3/12/2013 20:46'!
initialize
	super initialize.! !

!UserView methodsFor: 'testing' stamp: 'CameronRieck 3/7/2013 16:48'!
isUserView
	^ true! !

!UserView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/16/2013 15:40'!
justDroppedInto: aPotentialRoomView event: anEvent
	super justDroppedInto: aPotentialRoomView event: anEvent.
	aPotentialRoomView isRoomView
		ifTrue: [ 
			self user enterRoom: aPotentialRoomView room.
			previousOwner := aPotentialRoomView ]
		ifFalse: [ previousOwner := World ]! !

!UserView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/16/2013 15:39'!
justGrabbedFrom: formerOwner
	super justGrabbedFrom: formerOwner.
	formerOwner isRoomView
		ifTrue: [ self user exitCurrentRoom ].! !

!UserView methodsFor: 'dropping/grabbing' stamp: 'CameronRieck 3/16/2013 15:36'!
rejectDropMorphEvent: evt
	self formerOwner: previousOwner.
	super rejectDropMorphEvent: evt.! !

!UserView methodsFor: 'events' stamp: 'CameronRieck 3/12/2013 20:50'!
speakButtonPressed
	| dialog |
	dialog := TextEditorDialogWindow new.
	dialog title: 'Type a message to speak'.
	self openModal: dialog.
	dialog entryText ifNotNil: [ user speak: dialog entryText asString ].! !

!UserView methodsFor: 'model' stamp: 'CameronRieck 3/7/2013 16:54'!
user
	^ user! !

!UserView methodsFor: 'model' stamp: 'CameronRieck 3/12/2013 20:33'!
user: aUser
	user := aUser! !

!Morph methodsFor: '*CS520-HomeAutomation' stamp: 'CameronRieck 3/10/2013 00:00'!
isDeviceView
	^ false! !

!Morph methodsFor: '*CS520-HomeAutomation' stamp: 'CameronRieck 3/7/2013 17:34'!
isHomeAutomationView
	^ false! !

!Morph methodsFor: '*CS520-HomeAutomation' stamp: 'CameronRieck 3/7/2013 16:54'!
isRoomView
	^ false! !

!Morph methodsFor: '*CS520-HomeAutomation' stamp: 'CameronRieck 3/7/2013 16:48'!
isUserView
	^ false! !

Object subclass: #HomeAutomationDevice
	instanceVariableNames: 'handledEvents room name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

HomeAutomationDevice class
	instanceVariableNames: 'dummyDevice'!

!HomeAutomationDevice commentStamp: 'CameronRieck 3/16/2013 14:53' prior: 0!
I am the base class for all types of home automation devices

Instance Variables:
	handledEvents 				...
	room 						The room that the device is attached to.
	name 						The device name.

Class Instance Variables:
	dummyDevice				A dumb device that can be used for testing.!

HomeAutomationDevice class
	instanceVariableNames: 'dummyDevice'!

HomeAutomationDevice subclass: #ActivityTracker
	instanceVariableNames: 'usersTracked speechLog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!ActivityTracker class methodsFor: 'instance creation' stamp: 'CameronRieck 3/11/2013 18:50'!
createWithName: aName
	| newObserver |
	newObserver := ActivityTracker new.
	newObserver name: aName.
	^ newObserver.! !

!ActivityTracker methodsFor: 'users' stamp: 'CameronRieck 3/7/2013 13:26'!
detectedUser: aUser 
	^ usersTracked includes: aUser.	
! !

!ActivityTracker methodsFor: 'users' stamp: 'CameronRieck 3/11/2013 15:17'!
hasHeard: words from: aUser 
	^ (speechLog at: aUser ifAbsent: [ ^ false ]) includes: words.! !

!ActivityTracker methodsFor: 'initialize-release' stamp: 'CameronRieck 3/12/2013 20:54'!
initialize
	super initialize.
	usersTracked := Set new.
	speechLog := Dictionary new.
	! !

!ActivityTracker methodsFor: 'users' stamp: 'CameronRieck 3/7/2013 17:03'!
isTrackingUsers
	^ (usersTracked size > 0)! !

!ActivityTracker methodsFor: 'devices' stamp: 'CameronRieck 3/11/2013 11:23'!
observe: aDevice
	aDevice when: #userEntered send: #userEntered: to: self with: #aUser.
	aDevice when: #userExited send: #userExited: to: self with: #aUser.
	aDevice when: #userSpoke send: #userSpoke: to: self with: #userAndWords.! !

!ActivityTracker methodsFor: 'users' stamp: 'CameronRieck 3/11/2013 16:04'!
speechLog
	^ speechLog
	! !

!ActivityTracker methodsFor: 'devices' stamp: 'CameronRieck 3/7/2013 14:55'!
stopObserving: aDevice
	aDevice removeActionsWithReceiver: self. 
	! !

!ActivityTracker methodsFor: 'events' stamp: 'CameronRieck 3/12/2013 21:08'!
userEntered: aUser
	| statusMessage |
	usersTracked add: aUser.
	"self color: Color yellow. "
	statusMessage := aUser name , ' entered the ' , aUser room name.
	self triggerEvent: #updated with: statusMessage.! !

!ActivityTracker methodsFor: 'events' stamp: 'CameronRieck 3/12/2013 21:09'!
userExited: aUser
	| statusMessage |
	(usersTracked includes: aUser) ifTrue: [ usersTracked remove: aUser ].
	"(self isTrackingUsers) ifFalse: [ self color: (self defaultColor) ]."
	statusMessage := aUser name , ' exited the ' , aUser room name.
	self triggerEvent: #updated with: statusMessage.
	! !

!ActivityTracker methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 16:30'!
userSpoke: userAndWords
	| logEntry statusMessage user words |
	user := userAndWords key.
	words := userAndWords value.
	logEntry := speechLog at: user ifAbsent: [ speechLog at: user put: (LinkedList new) ].
	logEntry add: words.
	statusMessage := user name , ' said ' , words , ' in the ' , user room name.
	self triggerEvent: #updated with: statusMessage.! !

!ActivityTracker methodsFor: 'users' stamp: 'CameronRieck 3/10/2013 14:23'!
usersTracked
	^ usersTracked
	! !

HomeAutomationDevice subclass: #Camera
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!Camera class methodsFor: 'instance creation' stamp: 'CameronRieck 3/3/2013 12:02'!
createWithName: aName
	| newCamera |
	newCamera := Camera new.
	newCamera name: aName.
	^ newCamera.! !

!Camera methodsFor: 'rooms' stamp: 'CameronRieck 3/11/2013 15:57'!
attachTo: aRoom
	"We must simulate a user entering a room when the camera is installed in a room when users are already present.
	Otherwise, interested observers will not know who is in the room until they come back in."
	super attachTo: aRoom.
	aRoom usersPresent do: [ :each | self userEntered: each ].! !

!Camera methodsFor: 'initialize-release' stamp: 'CameronRieck 3/12/2013 21:04'!
initialize
	super initialize.
	self handleEvent: #userEntered with: (self userEnteredMessage).
	self handleEvent: #userExited with: (self userExitedMessage).! !

!Camera methodsFor: 'events' stamp: 'CameronRieck 3/10/2013 15:05'!
userEntered: aUser
	self triggerEvent: #userEntered with: aUser.! !

!Camera methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 16:29'!
userExited: aUser
	self triggerEvent: #userExited with: aUser.! !

!HomeAutomationDevice class methodsFor: 'instance creation' stamp: 'CameronRieck 3/13/2013 19:34'!
dummy
	dummyDevice
		ifNil: [ 
			dummyDevice := HomeAutomationDevice new.
			dummyDevice name: 'dummy' ].
	^ dummyDevice! !

!HomeAutomationDevice methodsFor: 'rooms' stamp: 'CameronRieck 3/10/2013 00:04'!
attachTo: aRoom
	room := aRoom.
	aRoom addDevice: self.
	self observeRoom: aRoom.! !

!HomeAutomationDevice methodsFor: 'rooms' stamp: 'CameronRieck 3/11/2013 13:35'!
detachFromRoom
	room removeDevice: self.
	self stopObservingRoom: room.
	room := Room noneRoom.! !

!HomeAutomationDevice methodsFor: 'events' stamp: 'CameronRieck 3/7/2013 13:36'!
handleEvent: anEvent with: messageParamAssoc
	handledEvents at: anEvent put: messageParamAssoc.! !

!HomeAutomationDevice methodsFor: 'events' stamp: 'CameronRieck 3/7/2013 13:38'!
handlesEvent: anEvent
	^ handledEvents includes: anEvent! !

!HomeAutomationDevice methodsFor: 'initialize-release' stamp: 'CameronRieck 3/11/2013 15:22'!
initialize
	super initialize.
	handledEvents := Dictionary new.
	room := Room noneRoom.! !

!HomeAutomationDevice methodsFor: 'rooms' stamp: 'CameronRieck 3/11/2013 13:40'!
isAttachedTo: aRoom 
	^ room = aRoom! !

!HomeAutomationDevice methodsFor: 'rooms' stamp: 'CameronRieck 3/11/2013 13:40'!
isAttachedToARoom
	^ (room = Room noneRoom) not.! !

!HomeAutomationDevice methodsFor: 'events' stamp: 'CameronRieck 3/7/2013 17:13'!
name 
	^ name! !

!HomeAutomationDevice methodsFor: 'properties' stamp: 'CameronRieck 3/7/2013 17:12'!
name: aByteString 
	name := aByteString.! !

!HomeAutomationDevice methodsFor: 'rooms' stamp: 'CameronRieck 3/10/2013 00:04'!
observeRoom: aRoom
	handledEvents
		associationsDo: [ :assoc | 
			| event message value |
			event := assoc key.
			message := assoc value key.
			value := assoc value value.
			aRoom
				when: event
				send: message
				to: self
				with: value ]! !

!HomeAutomationDevice methodsFor: 'rooms' stamp: 'CameronRieck 3/11/2013 20:31'!
room
	^ room! !

!HomeAutomationDevice methodsFor: 'rooms' stamp: 'CameronRieck 3/9/2013 23:59'!
stopObservingRoom: aRoom 
	aRoom removeActionsWithReceiver: self.! !

!HomeAutomationDevice methodsFor: 'users' stamp: 'CameronRieck 3/14/2013 21:55'!
userAllowed: aUser
	^ true! !

!HomeAutomationDevice methodsFor: 'events' stamp: 'CameronRieck 3/10/2013 15:04'!
userEnteredMessage
	^ (Association new) key: #userEntered: value: #aUser.
	! !

!HomeAutomationDevice methodsFor: 'events' stamp: 'CameronRieck 3/10/2013 15:04'!
userExitedMessage
	^ (Association new) key: #userExited: value: #aUser.
	! !

!HomeAutomationDevice methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 11:19'!
userSpokeMessage
	^ (Association new) key: #userSpoke: value: #userAndWords.
	! !

HomeAutomationDevice subclass: #HomeCommunicator
	instanceVariableNames: 'speakers usersTracked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!HomeCommunicator class methodsFor: 'instance creation' stamp: 'CameronRieck 3/11/2013 19:54'!
createWithName: aName
	| newObserver |
	newObserver := HomeCommunicator new.
	newObserver name: aName.
	^ newObserver.! !

!HomeCommunicator methodsFor: 'initialize-release' stamp: 'CameronRieck 3/12/2013 21:04'!
initialize
	super initialize.
	speakers := Set new.
	usersTracked := Set new.! !

!HomeCommunicator methodsFor: 'devices' stamp: 'CameronRieck 3/11/2013 19:53'!
observe: aDevice
	aDevice when: #userEntered send: #userEntered: to: self with: #aUser.
	aDevice when: #userExited send: #userExited: to: self with: #aUser.
	aDevice when: #userSpoke send: #userSpoke: to: self with: #userAndWords.! !

!HomeCommunicator methodsFor: 'devices' stamp: 'CameronRieck 3/11/2013 19:54'!
useSpeaker: aSpeaker 
	speakers add: aSpeaker.! !

!HomeCommunicator methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 20:10'!
userEntered: aUser 
	usersTracked add: aUser.! !

!HomeCommunicator methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 22:11'!
userExited: aUser 
	(usersTracked includes: aUser) ifTrue: [ usersTracked remove: aUser ].! !

!HomeCommunicator methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 22:16'!
userSpoke: userAndWords
	| fromUser words subStrings toUserName toUser toSpeaker |
	fromUser := userAndWords key.
	words := userAndWords value.

	"pull out name in words"
	subStrings := words subStrings: ':'.
	(subStrings size > 1) ifFalse: [ ^ self ].
	toUserName := subStrings at: 1.

	"find user based on name (no caps)"
	toUser := nil.
	usersTracked do: [ :each | (toUserName asLowercase = each name asLowercase) ifTrue: [ toUser := each ] ].
	toUser ifNil: [ ^ self ].

	"find speaker based on room"
	toSpeaker := nil.
	speakers do: [ :each | (each room = toUser room) ifTrue: [ toSpeaker := each ] ].
	toSpeaker ifNil: [ ^ self ].
	
	"tell speaker to broadcast words"
	toSpeaker broadcast: words.! !

HomeAutomationDevice subclass: #Microphone
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!Microphone class methodsFor: 'instance creation' stamp: 'CameronRieck 3/11/2013 16:01'!
createWithName: aName
	| newMicrophone |
	newMicrophone := Microphone new.
	newMicrophone name: aName.
	^ newMicrophone.! !

!Microphone methodsFor: 'initialize-release' stamp: 'CameronRieck 3/12/2013 21:04'!
initialize
	super initialize.
	self handleEvent: #userSpoke with: (self userSpokeMessage).! !

!Microphone methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 17:26'!
userSpoke: userAndWords
	self triggerEvent: #userSpoke with: userAndWords.! !

HomeAutomationDevice subclass: #SmartDoor
	instanceVariableNames: 'allowedUsers locked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!SmartDoor class methodsFor: 'instance creation' stamp: 'CameronRieck 3/14/2013 21:28'!
createWithName: aName 
	| newDevice |
	newDevice := SmartDoor new.
	newDevice name: aName.
	^ newDevice.! !

!SmartDoor methodsFor: 'state changing' stamp: 'CameronRieck 3/14/2013 21:56'!
allowUser: aUser
	allowedUsers add: aUser.! !

!SmartDoor methodsFor: 'users' stamp: 'CameronRieck 3/14/2013 21:34'!
allowedUsers
	^ allowedUsers! !

!SmartDoor methodsFor: 'testing' stamp: 'CameronRieck 3/15/2013 18:54'!
ifLocked: lockedBlock ifUnlocked: unlockedBlock 
	self isLocked ifTrue: [ ^ lockedBlock value ] ifFalse: [ ^ unlockedBlock value ].! !

!SmartDoor methodsFor: 'initialize-release' stamp: 'CameronRieck 3/14/2013 21:34'!
initialize
	super initialize.
	allowedUsers := Set new.
	locked := false.! !

!SmartDoor methodsFor: 'testing' stamp: 'CameronRieck 3/14/2013 21:39'!
isLocked
	^ locked! !

!SmartDoor methodsFor: 'testing' stamp: 'CameronRieck 3/14/2013 21:34'!
isUnlocked
	^ locked not! !

!SmartDoor methodsFor: 'state changing' stamp: 'CameronRieck 3/15/2013 18:51'!
lock
	locked := true.
	allowedUsers addAll: room usersPresent.
	self triggerEvent: #updated.! !

!SmartDoor methodsFor: 'state changing' stamp: 'CameronRieck 3/15/2013 18:51'!
unlock
	allowedUsers removeAll.
	locked := false.
	self triggerEvent: #updated.! !

!SmartDoor methodsFor: 'state changing' stamp: 'CameronRieck 3/14/2013 22:24'!
userAllowed: aUser
	^ (self isUnlocked) | (allowedUsers includes: aUser)! !

HomeAutomationDevice subclass: #SmartLight
	instanceVariableNames: 'usersInRoom on'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!SmartLight class methodsFor: 'instance creation' stamp: 'CameronRieck 3/13/2013 19:31'!
createWithName: aName
	| newLight |
	newLight := SmartLight new.
	newLight name: aName.
	^ newLight.! !

!SmartLight methodsFor: 'rooms' stamp: 'CameronRieck 3/13/2013 19:25'!
attachTo: aRoom
	super attachTo: aRoom.
	aRoom usersPresent do: [ :each | self userEntered: each ].! !

!SmartLight methodsFor: 'testing' stamp: 'CameronRieck 3/13/2013 19:54'!
ifOn: onBlock ifOff: offBlock 
	self isOn ifTrue: [ ^ onBlock value ].
	self isOn ifFalse: [ ^ offBlock value ].! !

!SmartLight methodsFor: 'initialize-release' stamp: 'CameronRieck 3/13/2013 19:37'!
initialize
	super initialize.
	usersInRoom := Set new.
	on := false.! !

!SmartLight methodsFor: 'testing' stamp: 'CameronRieck 3/13/2013 19:35'!
isOff
	^ self isOn not! !

!SmartLight methodsFor: 'testing' stamp: 'CameronRieck 3/13/2013 19:38'!
isOn
	^ usersInRoom size > 0 ! !

!SmartLight methodsFor: 'devices' stamp: 'CameronRieck 3/13/2013 19:23'!
observe: aDevice
	aDevice when: #userEntered send: #userEntered: to: self with: #aUser.
	aDevice when: #userExited send: #userExited: to: self with: #aUser.! !

!SmartLight methodsFor: 'events' stamp: 'CameronRieck 3/13/2013 19:41'!
userEntered: aUser 
	usersInRoom add: aUser.
	self triggerEvent: #updated.! !

!SmartLight methodsFor: 'events' stamp: 'CameronRieck 3/13/2013 19:42'!
userExited: aUser 
	usersInRoom remove: aUser ifAbsent: [ ^ self ].
	self triggerEvent: #updated.! !

HomeAutomationDevice subclass: #SmartThermostat
	instanceVariableNames: 'usersInRoom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!SmartThermostat class methodsFor: 'instance creation' stamp: 'CameronRieck 3/13/2013 20:13'!
createWithName: aName
	| newDevice |
	newDevice := SmartThermostat new.
	newDevice name: aName.
	^ newDevice.! !

!SmartThermostat methodsFor: 'temperature' stamp: 'CameronRieck 3/13/2013 20:28'!
averageUserTemperature
	| total |
	total := 0.
	usersInRoom do: [ :each | total := total + each preferredTemperature ].
	^ (total / (usersInRoom size)) asInteger ! !

!SmartThermostat methodsFor: 'temperature' stamp: 'CameronRieck 3/13/2013 20:25'!
defaultTemperature
	^ 60! !

!SmartThermostat methodsFor: 'initialize-release' stamp: 'CameronRieck 3/13/2013 20:16'!
initialize
	super initialize.
	usersInRoom := Set new.! !

!SmartThermostat methodsFor: 'devices' stamp: 'CameronRieck 3/13/2013 20:21'!
observe: aDevice
	aDevice when: #userEntered send: #userEntered: to: self with: #aUser.
	aDevice when: #userExited send: #userExited: to: self with: #aUser.! !

!SmartThermostat methodsFor: 'temperature' stamp: 'CameronRieck 3/13/2013 20:26'!
temperature
	(usersInRoom size = 0) ifTrue: [ ^ self defaultTemperature ] ifFalse: [ ^ self averageUserTemperature ].
	! !

!SmartThermostat methodsFor: 'events' stamp: 'CameronRieck 3/13/2013 21:43'!
userEntered: aUser 
	usersInRoom add: aUser.
	self triggerEvent: #updated with: 'User entered'! !

!SmartThermostat methodsFor: 'events' stamp: 'CameronRieck 3/13/2013 21:43'!
userExited: aUser 
	usersInRoom remove: aUser ifAbsent: [ ^ self ].
	self triggerEvent: #updated with: 'User exited'.! !

HomeAutomationDevice subclass: #Speaker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!Speaker class methodsFor: 'instance creation' stamp: 'CameronRieck 3/11/2013 18:14'!
createWithName: aName
	| newSpeaker |
	newSpeaker := Speaker new.
	newSpeaker name: aName.
	^ newSpeaker.! !

!Speaker methodsFor: 'events' stamp: 'CameronRieck 3/11/2013 18:59'!
broadcast: aByteString
	| deviceAndWords |
	deviceAndWords := Association key: self value: aByteString.
	self triggerEvent: #speakerNoise with: deviceAndWords.! !

!Speaker methodsFor: 'initialize-release' stamp: 'CameronRieck 3/12/2013 21:04'!
initialize
	super initialize.! !

Object subclass: #Room
	instanceVariableNames: 'attachedDevices name usersPresent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

Room class
	instanceVariableNames: 'noneRoom'!

Room class
	instanceVariableNames: 'noneRoom'!

!Room class methodsFor: 'instance creation' stamp: 'CameronRieck 3/12/2013 19:35'!
createWithName: aName
	| room |
	room := Room new.
	room setName: aName.
	^ room.! !

!Room class methodsFor: 'instance creation' stamp: 'CameronRieck 3/12/2013 19:35'!
noneRoom
	(noneRoom = nil) ifTrue: [ noneRoom := Room createWithName: 'None' ].
	^ noneRoom.! !

!Room methodsFor: 'devices' stamp: 'CameronRieck 3/11/2013 18:43'!
addDevice: aDevice
	attachedDevices add: aDevice.
	aDevice when: #speakerNoise send: #speakerMadeNoise: to: self with: #deviceAndWords.! !

!Room methodsFor: 'users' stamp: 'CameronRieck 3/15/2013 19:44'!
addUser: aUser
	(self userIsAllowedToEnter: aUser)
		ifTrue: [ 
			usersPresent add: aUser.
			aUser room: self.
			self triggerEvent: #userEntered with: aUser.
			aUser
				when: #userSpoke
				send: #userSpoke:
				to: self
				with: #userAndWords.
			^ true ]
		ifFalse: [ ^ false ].
			! !

!Room methodsFor: 'devices' stamp: 'CameronRieck 3/6/2013 18:51'!
attachedDevices
	^ attachedDevices! !

!Room methodsFor: 'devices' stamp: 'CameronRieck 3/6/2013 18:51'!
devices
	^ attachedDevices! !

!Room methodsFor: 'devices' stamp: 'CameronRieck 3/3/2013 11:48'!
hasDevice: aDevice
	^ attachedDevices includes: aDevice! !

!Room methodsFor: 'users' stamp: 'CameronRieck 3/6/2013 20:02'!
includesUser: aUser 
	^ usersPresent includes: aUser.! !

!Room methodsFor: 'initialize-release' stamp: 'CameronRieck 3/6/2013 20:01'!
initialize
	attachedDevices := OrderedCollection new.
	usersPresent := Set new.! !

!Room methodsFor: 'properties' stamp: 'CameronRieck 3/3/2013 11:54'!
name
	^ name! !

!Room methodsFor: 'devices' stamp: 'CameronRieck 3/9/2013 23:48'!
removeDevice: aDevice
	self triggerEvent: #deviceRemoved with: aDevice.
	(attachedDevices includes: aDevice) ifTrue: [ attachedDevices remove: aDevice ].
	! !

!Room methodsFor: 'users' stamp: 'CameronRieck 3/14/2013 22:13'!
removeUser: aUser
	self triggerEvent: #userExited with: aUser.
	aUser removeActionsWithReceiver: self.
	usersPresent remove: aUser ifAbsent: [].
	! !

!Room methodsFor: 'properties' stamp: 'CameronRieck 3/3/2013 11:54'!
setName: aName
	name := aName.! !

!Room methodsFor: 'users' stamp: 'CameronRieck 3/11/2013 18:44'!
speakerMadeNoise: deviceAndWords
	self triggerEvent: #speakerMadeNoise with: deviceAndWords.
! !

!Room methodsFor: 'users' stamp: 'CameronRieck 3/15/2013 19:44'!
userIsAllowedToEnter: aUser
	^ (attachedDevices allSatisfy: [ :each | each userAllowed: aUser ])! !

!Room methodsFor: 'users' stamp: 'CameronRieck 3/11/2013 11:10'!
userSpoke: userAndWords
	self triggerEvent: #userSpoke with: userAndWords.! !

!Room methodsFor: 'users' stamp: 'CameronRieck 3/9/2013 23:50'!
usersPresent
	^ usersPresent! !

Object subclass: #User
	instanceVariableNames: 'name room lastNoiseHeard lastNoiseHeardSource preferredTemperature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS520-HomeAutomation'!

!User class methodsFor: 'instance creation' stamp: 'CameronRieck 3/14/2013 22:05'!
createWithName: aName 
	| newUser |
	newUser := User new.
	newUser name: aName.
	newUser room: Room noneRoom.
	^ newUser.! !

!User methodsFor: 'actions' stamp: 'CameronRieck 3/14/2013 22:00'!
enterRoom: aRoom
	aRoom addUser: self.
	room = aRoom
		ifTrue: [ 
			room
				when: #speakerMadeNoise
				send: #speakerMadeNoise:
				to: self
				with: #deviceAndWords ]! !

!User methodsFor: 'actions' stamp: 'CameronRieck 3/11/2013 18:22'!
exitCurrentRoom
	room removeUser: self.
	room removeActionsWithReceiver: self.
	self enterRoom: Room noneRoom.
	! !

!User methodsFor: 'initialize-release' stamp: 'CameronRieck 3/14/2013 22:05'!
initialize
	super initialize.
	self name: 'User'.
	self preferredTemperature: 68.
	self room: Room noneRoom.! !

!User methodsFor: 'accessing' stamp: 'CameronRieck 3/11/2013 18:18'!
lastNoiseHeard
	^ lastNoiseHeard! !

!User methodsFor: 'accessing' stamp: 'CameronRieck 3/11/2013 18:24'!
lastNoiseHeardSource
	^ lastNoiseHeardSource! !

!User methodsFor: 'properties' stamp: 'CameronRieck 3/7/2013 15:19'!
name
	^ name! !

!User methodsFor: 'properties' stamp: 'CameronRieck 3/6/2013 20:00'!
name: aName 
	name := aName.! !

!User methodsFor: 'accessing' stamp: 'CameronRieck 3/13/2013 20:28'!
preferredTemperature
	^ preferredTemperature! !

!User methodsFor: 'accessing' stamp: 'CameronRieck 3/13/2013 20:22'!
preferredTemperature: aTemperature
	preferredTemperature := aTemperature! !

!User methodsFor: 'properties' stamp: 'CameronRieck 3/10/2013 17:28'!
room
	^ room! !

!User methodsFor: 'properties' stamp: 'CameronRieck 3/14/2013 22:07'!
room: aRoom
	room := aRoom! !

!User methodsFor: 'actions' stamp: 'CameronRieck 3/11/2013 21:55'!
speak: words
	| userAndWords |
	userAndWords := (Association key: self value: words asString).
	self triggerEvent: #userSpoke with: userAndWords.! !

!User methodsFor: 'properties' stamp: 'CameronRieck 3/11/2013 18:45'!
speakerMadeNoise: deviceAndWords
	lastNoiseHeard := deviceAndWords value.
	lastNoiseHeardSource := deviceAndWords key.! !
